#!/usr/bin/python

# $Id: fedora-setup.py,v 1.12 2004/06/23 12:33:07 adrian Exp $
#
# select a mirror
# Copyright (C) 2004 Adrian Reber

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

import sys
import os
import commands
import threading
import urllib
import gobject
import gtk
import getopt
import urllib
import string
import time
from gtk import *

version = "0.1"
name = "mirror-setup"
result = "NORESULT"
sema4 = threading.Semaphore()
fn= ""
netselect="/usr/bin/netselect"
mirrorlist = "http://lisas.de/~adrian/fedora-mirrors.list"
display_mirrors = "3"

class SourcesListWriter:

	def __init__(self, fn=None):
		if fn is None:
			self.fn = "sources.list"
		else:
			self.fn = fn
		self.list = {}
		self.priority = {}

	def add(self,key,item,priority):
		self.list[key] = item
		prio_str = "%d" % priority
		self.priority[key] = prio_str

	def remove(self,key):
		del self.list[key]
		del self.priority[key]

	def write(self):
		fp=open(self.fn,"w")
		currtime = time.strftime("%x %X %Z", time.localtime())
		fp.write("### generated from %s ###\n" % name)
		fp.write("### %(currtime)s\n\n" % vars())
		#find lowest priority
		priority = {}
		for key in self.priority.keys():
			temp = string.atoi(self.priority[key])
			priority[temp] = key
		thekeys = priority.keys()
		thekeys.sort()
		for key in thekeys:
			real_key = priority[key]
			fp.write("### " +  self.list[real_key] + "\n")
			fp.write("rpm " + real_key + "\n\n")
		fp.close()

class Mirror:

	def __init__(self):
		self.url = ''
		self.comment = ''

	def setUrl(self,url):
		self.url = url

	def setComment(self,comment):
		self.comment = comment

	def getUrl(self):
		return self.url

	def getComment(self):
		return self.comment


class Mirrors:

	def __init__(self,data):
		self.mirrors = list()
		self.parse(data)
	
	def parse(self,data):
		found = 0
		comment = ''
		url = ''
		for line in data:
			if (line=="Fedora Extras"):
				found = 1
				continue
			if (found==1 and comment==''):
				comment = line
				continue
			if (found==1 and url==''):
				url = line
				continue
			if (found==1):
				self.addMirror(comment,url)
				comment=''
				url=''
				
				
		
	def addMirror(self,comment,url):
		mirror = Mirror()
		mirror.setUrl(url)
		mirror.setComment(comment)
		self.mirrors.append(mirror)

	def getSize(self):
		return len(self.mirrors)

	def getUrl(self,index):
		return self.mirrors[index].getUrl()

	def getComment(self,index):
		return self.mirrors[index].getComment()

class NetSelect(threading.Thread):

	def __init__(self, hosts):
		threading.Thread.__init__(self)
		self.hosts = hosts

	def run(self):
		sema4.acquire()
		global result
		if (os.path.isfile(netselect)):
			command = netselect + " -s " + display_mirrors + " "
			for key in self.hosts.keys():
				command += key + " "
			output = commands.getoutput(command)
			result = output
		else:
			result = netselect + " not found!"
		sema4.release()

class MirrorSelector:

	counter = 0
	netselect_text = "Searching for fastest mirror\n"

	def check_thread(self):
		if (self.counter >=7):
			self.counter = 0
		global result
		self.message.label.set_text(self.netselect_text + "Please wait" + ". " * self.counter)
		self.counter = self.counter+1
		if (sema4.acquire(FALSE)==FALSE):
			return TRUE
		sema4.release()
		self.counter = 0
		localresult = string.split(result,"\n")
		self.ok_button.set_sensitive(TRUE)
		label = self.netselect_text + "Displaying the %d fastest mirrors:\n" % len(localresult)
		i = 0
		for line in localresult:
			button = gtk.CheckButton()
			try:
				button = self.checkbuttons[self.parameter[line[6:]]]
				i = i + 1
				self.priority[line[6:]] = i
				button.set_active(gtk.TRUE)
				label += line[6:] + "\n"
			except KeyError:
				label = self.netselect_text + result
		self.message.label.set_text(label)
		result = "NORESULT"
		return FALSE

	def netselect(self, widget, data=None):
		for key in self.priority.keys():
			self.priority[key] = 0
		if (sema4.acquire(FALSE)==FALSE):
			return
		sema4.release()
		net_select = NetSelect(self.parameter)
		net_select.start()
		gobject.timeout_add(200,self.check_thread)
		self.message.label.set_text(self.netselect_text)
		self.ok_button.set_sensitive(FALSE)
		self.message.run()
		self.message.hide()

	def write(self, widget, data=None):
		self.writer.write()

	def checked(self, widget, event, data):
		if (widget.get_active()):
			url = string.split(self.mirrors.getUrl(data),'/')
			self.writer.add(self.mirrors.getUrl(data),self.mirrors.getComment(data),
				self.priority[url[2]])
		else:
			self.writer.remove(self.mirrors.getUrl(data))

	def delete_event(self, widget, event, data=None):
		return gtk.FALSE

	def destroy(self, widget, data=None):
		gtk.main_quit()
	
	def create_table(self):
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.vbox.add(scrolled_window)
		table = gtk.Table(self.mirrors.getSize(),2,gtk.FALSE)
		label = gtk.Label("Fedora Mirror")
		table.attach(label, 0, 1, 0, 1)
		label = gtk.Label("choose")
		table.attach(label, 1, 2, 0, 1,0,0,5,5)
		for x in range(self.mirrors.getSize()):
			text = string.replace(mirrors.getComment(x),'(mirror','\n(mirror')	
			url = string.split(mirrors.getUrl(x),'/')
			self.parameter[url[2]] = x
			self.priority[url[2]] = 0
			label = gtk.Label(text)
			label.set_justify(gtk.JUSTIFY_CENTER)
			table.attach(label, 0, 1, x+1, x+2, 0, 0,3,3)
			button = gtk.CheckButton()
			button.set_active(gtk.FALSE)
			table.attach(button,1,2,x+1, x+2, 0 ,0,3,3)
			button.connect("clicked",self.checked, None,x)
			self.checkbuttons[x] = button
		scrolled_window.add_with_viewport(table)
		scrolled_window.set_size_request(500,300)
		
	def add_buttons(self):
		buttonbox = gtk.HButtonBox()
		self.vbox.add(buttonbox)
		self.measure_tip = gtk.Tooltips()
		measure = gtk.Button("Measure")
		write = gtk.Button("Write")
		exit = gtk.Button("Exit")
		exit.connect_object("clicked", gtk.Widget.destroy, self.window)
		write.connect("clicked",self.write, None)
		measure.connect("clicked",self.netselect, None)
		buttonbox.pack_start(measure)
		buttonbox.pack_start(write)
		buttonbox.pack_start(exit)
		self.measure_tip.set_tip(measure,"Measure which mirror is the closest")
		self.measure_tip.set_tip(exit,"Quit from %s" % name)
		buttonbox.set_layout(gtk.BUTTONBOX_SPREAD)

		
	def __init__(self, mirrors):
		self.checkbuttons = {}
		self.parameter = {}
		self.priority = {}
		self.mirrors = mirrors
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)
		self.message = gtk.MessageDialog(self.window,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_INFO,gtk.BUTTONS_NONE)
		self.ok_button = self.message.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		self.vbox = gtk.VBox()
		self.vbox.set_homogeneous(FALSE)
		self.window.add(self.vbox)
		self.create_table()
		self.add_buttons()
		self.window.show_all()
		self.writer = SourcesListWriter(fn)
		self.window.set_title(name)

	def main(self):
		gtk.gdk.threads_init()
		gtk.main()

userknowsitbetter=0

def weknowitbetter():
	if (userknowsitbetter==0):
		print "This program should be run as root."
		print "But if you think you know what you are doing"
		print "you can specify '--iknowitbetter' on the command line."
		sys.exit(1)
	else:
		print "So you think you know it better..."
		print "Okay, let's see if you really do"
		print "Running as non root user..."

def usage():
	print >> sys.stderr, """
Usage: %s [options] [file]

Options:
  -h, --help         print this message
  -m, --mirrorlist   URL of the mirrorr list
                     defaults to:
		     %s
  -n, --number       specify the number of mirrors used
                     defaults to: %d

Argumemt:
  file               file for writing output
                     default: ./sources.list
		     file will be overwritten
""" % (name, mirrorlist, display_mirrors)

print >> sys.stderr, """%s Release %s - Copyright (C) 2004 Adrian Reber
%s comes with ABSOLUTELY NO WARRANTY - for details read the license.
""" % (name, version, name)


try:
	o, a = getopt.getopt(sys.argv[1:], 'm:hn:',['iknowitbetter','help','mirrorlist=','number='])
	opts = {}
except:
	usage()
	sys.exit(0)

for k,v in o:
	opts[k] = v
if (opts.has_key("-h") or opts.has_key("--help")):
	usage()
	sys.exit(0)
if (opts.has_key("-n") or opts.has_key("--number")):
	display_mirrors=opts[k]
if (opts.has_key("-m") or opts.has_key("--mirrorlist")):
	mirrorlist=opts[k]

if opts.has_key('--iknowitbetter'):
	userknowsitbetter=1

if len(a) < 1:
	fn = "sources.list"
else:
	fn=a[0]

if (os.getuid()!=0):
	weknowitbetter()

try:
	f = urllib.urlopen(mirrorlist)
except IOError, (errno, strerror):
	print "I/O error(%s): %s\n" % (errno, strerror)
	print "Maybe you need to specify a proxy:"
	print "export http_proxy=\"http://proxy:port/\"\n"
	sys.exit(1)
	
data = string.split(f.read(), '\n')

mirrors = Mirrors(data)

mirrorSelector = MirrorSelector(mirrors)
try:
	mirrorSelector.main()
except KeyboardInterrupt:
	print "haha"
